#
# bfp_pgm_lib.pl - various tables used by blast_pan
#
#   $Name: HEAD $ - $Id: bfp_pgm_lib.pl,v 1.5 2011/02/21 16:46:21 wrp Exp $
#
#   copyright (c) 2000 by William R. Pearson and the U. of Virginia
#

### default extension for blast output
%def_res_ext = ( blastp => 'bp',
		 blastn => 'bn',
		 tblastn => 'tbn',
		 blastx => 'bx',
		 fasta => 'ok2',
		 fastn=> 'ok6',
		 ssearch => 'ss',
		 tfastx => 'tx2',
		 );

%fasta_pgm = ( fasta => 'fasta36_t',
	       fastn => 'fasta36_t -n',
	       ssearch => 'ssearch36_t',
	       tfastx => 'tfastx36_t',
	       );

### fasta program name for re-search
%r_pgm_n = ( blastp => 'fasta',
	     blastn => 'fasta',
	     tblastn => 'fastx',
	     blastx => 'fasta',
	     fasta => 'fasta',
	     fastn => 'fasta',
	     ssearch => 'ssearch',
	     tfastx => 'fastx',
	     pvcompfa => 'fasta',
	     pvcompsw => 'ssearch',
	     pvcomptfx => 'fastx',
);

### fasta program searchfa.cgi definition for re-search
%r_pgm_a = ( blastp => 'fa',
	     blastn => 'fa',
	     tblastn => 'fx',
	     blastx => 'fa',
	     fasta => 'fa',
	     fastn => 'fa',
	     ssearch => 'sw',
	     tfastx => 'fx',
	     pvcompfa => 'fa',
	     pvcomptfx => 'fx',
	     pvcompsw => 'sw',
);

### database type for re-search - used by searchfx, old www entrez
%def_db_type = ( blastp => 'p',
		 blastn => 'n',
		 tblastn => 'n',
		 blastx => 'p',
		 fasta => 'p',
		 ssearch => 'p',
		 tfastx => 'n',
		 pvcompfa => 'p',
		 pvcomptfx => 'n',
		 pvcompsw => 'p',
);

### database type for re-search - used by new www Entrez
%def_db_type_n = ( blastp => 'Protein',
		   blastn => 'Nucleotide',
		   tblastn => 'Nucleotide',
		   blastx => 'Protein',
		   fasta => 'Protein',
		   ssearch => 'Protein',
		   tfastx => 'Nucleotide',
		   pvcompfa => 'Protein',
		   pvcomptfx => 'Nucleotide',
		   pvcompsw => 'Protein',
		   );

%entrez_dopt = ( 'Protein'=>'GenPept',
		  'Nucleotide'=>'GenBank',
		  );

%def_gi_list = ( 0=>'(None)',
		 At=>'Arabidopsis thaliana',
		 Bs=>'Bacillus subtilis',
		 cow=>'Bos taurus',
		 Ce =>'Caenorhabditis elegans',
		 worm => 'Caenorhabditis elegans',
		 zebrafish =>'Danio rerio',
		 slime_mold =>'Dictyostelium discoideum',
		 fly => 'Drosophila melanogaster',
		 coli => 'Escherichia coli',
		 chicken => 'Gallus gallus',
		 chick => 'Gallus gallus',
		 human => 'Homo sapiens',
		 HIV => 'Human immunodeficiency virus type 1',
		 mouse => 'Mus musculus',
		 rabbit => 'Oryctolagus cuniculus',
		 rice => 'Oryza sativa',
		 pig => 'Ovis aries',
		 Pf => 'Plasmodium falciparum',
		 rat => 'Rattus norvegicus',
		 Sc => 'Saccharomyces cerevisiae',
		 yeast => 'Saccharomyces cerevisiae',
		 Sp => 'Schizosaccharomyces pombe',
		 SIV => 'Simian immunodeficiency virus',
		 frog => 'Xenopus laevis',
		 corn => 'Zea mays');

%bl_db_list = (
    nr => 1,
    drosoph => 1,
    month => 1,
    swissprot => 1,
    dbest => 1,
    dbsts => 1,
    est_mouse => 1,
    est_human => 1,
    est_others => 1,
    pdb => 1,
    pat => 1,
    vector => 1,
    kabat => 1,
    mito => 1,
    alu => 1,
    epd => 1,
    yeast => 1,
    ecoli => 1,
    gss => 1,
    htgs => 1,
);

################################################################

use HTTP::Request::Common qw(POST);
use LWP::UserAgent;
use LWP::Simple;

$ua = LWP::UserAgent->new;
$ua->agent( $0 );
$ua->from('wrp@virginia.edu');

$BLAST_RID="http://www.ncbi.nlm.nih.gov/blast/Blast.cgi?CMD=Get&FORMAT_TYPE=Text&RID=";

$check="CHECKED";

sub parse_search_opt {

###Set the default output file name
    unless ($opt_pgm) {
	$opt_pgm = $def_pgm;
	print STDERR "using: $opt_pgm\n";
    }

    if ($opt_pgm =~ /comp/) {
	$opt_r_data ||= "";
	$opt_r_ext = "comp";
    }

###Set the default output file name
    unless ( $opt_r_ext || $opt_pgm =~ /comp/) {
	$opt_r_ext = $def_res_ext{$opt_pgm} ||
	    die "No extension for $opt_pgm\n"
    }

    $opt_pgm_opts ||= " ";
    $opt_pgm_opts = " ".$opt_pgm_opts;

### set library for initial search
    $opt_slib ||= $def_slib;
    if ($opt_pgm =~ /blast/) {
	unless (exists($bl_db_list{$opt_slib})) {
	    print STDERR "\n*** Blast database: \"$opt_slib\" not recognized, try: \n";
	    foreach $org ( sort keys %bl_db_list ) {
		print STDERR " $org\n";
	    }
	    die "\n";
	}
    }

    if (defined ($opt_bl_org)) {
	if (exists($def_gi_list{$opt_bl_org})) {
	    $bl_gi_list = $def_gi_list{$opt_bl_org};
	}
	elsif ($opt_pgm =~ /blast/) {
	    print STDERR "\n *** Organism: \"$opt_bl_org\" not recognized, try:\n";
	    foreach $org ( sort keys %def_gi_list ) {
		print STDERR " $org ($def_gi_list{$org})\n";
	    }
	    die "\n";
	}
	else {
	    $bl_gi_list = $opt_bl_org;
	}
    }
    else {
	if ($opt_pgm =~ /blast/) {$bl_gi_list = $def_gi_list{'0'};}
	else {$bl_gi_list = "";}
	$opt_bl_org ||= "";	
    }

    if ($opt_verbose) {print STDERR "*** bl_gi_list: $bl_gi_list\n";}

    $opt_bl_org_text ||= "";

    if ( $opt_r_ext !~ /\./) {$opt_r_ext = ".".$opt_r_ext;}

    $opt_r_dir ||="./";
    if ($opt_r_dir !~ /\/$/) { $opt_r_dir .= "/";}
}


################################################################

sub get_blast_RID {

    my ($pgm, $search_seq, $opt_slib, $opt_max_e, $bl_gi_list, $opt_bl_org_text)  = @_;


#    print STDERR "$N_RES\t$N_ALN\n";

    $opt_slib = 'swissprot';

    my $req = POST 'http://www.ncbi.nlm.nih.gov/blast/Blast.cgi',
                      [ CMD=>'Put',
			PROGRAM => $opt_pgm,
			DATABASE => $opt_slib,
			QUERY => $search_seq,
			EXPECT => $opt_max_e,
#			LIST_ORG => $opt_bl_org_text,
#			GI_LIST => $bl_gi_list
			];

    my $res = $ua->request($req);

#    $QUEUE_OUT=(get("http://www.ncbi.nlm.nih.gov/blast/blast.cgi\?PROGRAM=$comp_pgm\&DATALIB=$opt_slib\&INPUT_TYPE=FASTA\&SEQUENCE=\"$search_seq\"\&EXPECT=$opt_minh\&DESCRIPTIONS=$N_RES\&ALIGNMENTS=$N_RES"));

#    print STDERR $res->as_string();

    my ($tmp) = ($res->as_string() =~ /^<!--QBlastInfoBegin\n\s+RID\s*=\s*(\S+)/ms);

    return $tmp;
}


sub get_RID_res {   # array of RID's

    my @RID_array = @_;
    my $current_RID;
    my $n = 0;
    my $max_n = scalar @_;

    if (!defined $N_RES) {$N_RES = 500;}
    if (!defined $N_ALN) {$N_ALN = 500;}

    $time = 2;
    $n = $max_n;

    while (@RID_array > 0) {

	$current_RID = shift @RID_array;

#	print STDERR " checking $current_RID\n";
#	print STDERR "$BLAST_RID$current_RID\&NCBI_GI=$check\&ALIGNMENTS=$N_RES\n";

	$RESULT=get("$BLAST_RID$current_RID\&NCBI_GI=$check\&ALIGNMENTS=$N_RES");

#	print $RESULT;

	if ($RESULT =~ /Status=WAITING/m) {
	    push @RID_array, $current_RID;
	    $n--;
	    sleep(1);
	}
	else {
	    $time = 2;
	    $res_name = $RID_seq{$current_RID};
	    $res_fname = $opt_r_dir.$res_name.$opt_r_ext;
	    print STDERR "$res_fname written\n";
	    open(R_FH,">$res_fname");
	    print R_FH $RESULT;
	    close(R_FH);
	    $n = scalar @RID_array;
	    if ($n == 0) {last;}
#	    $seq_RESULTS[$RID_hash{$current_RID}]=$RESULT;
	}

	if ( $n <= 0 ) {
	    sleep ($time);
	    print STDERR "waiting(time=$time)\n";
	    if ($time < 128) {$time *= 2;}
	} 
    }
}

1;

