#!/usr/local/bin/perl5
#
#  this page should parse (or send on) the values from blast_pan.html
#  variable  values
#  PROGRAM - blastp/tblastn -> --pgm 
#  DATALIB - blast databases -> --slib
#  input_format - FASTA_Color/GI_ColorNum
#  inputmode - input/upload (radio button)
#  query_list - <select> (textarea) -
#  uploadfile - (file)
#  color_scheme - <select>/0-5  -> --col
#  bfp_program - BP/BFP
#  GI_LIST - (blast orgs) -> --bl_org
#  LIST_ORG - (textarea) -> --bl_org_text
#  CONFIRM - (check box)
#  uploaddata - (file)
#  opt_mint - 1,2,4,6,8,10,20 -> --mint
#  opt_sqid - 80.0 -> 100.0 -> --sqid
#  opt_id_cut - 80.0 -> 100.0 -> --sqid
#  EXPECT 0.0001 - 100 -> --max_e
#

use Getopt::Long;
use LWP::Simple;   
use POSIX qw(setsid);

# needed for ps2pdf/gs
$ENV{PATH}="$ENV{PATH}:/usr/local/bin";

use CGI;
$q = new CGI;

##################################################################
## -- edit this http link line:
##################################################################
$http_fasta_loc="<a HREF=\"http://fasta.bioch.virginia.edu/fasta/cgi/";
$http_fastpan="<HR><B>Push the BACK button on your browser to go back.";

open (STDERR, ">&STDOUT");

GetOptions("Debug:s");

$opt_Debug ||= 0;

if ( ! $opt_Debug ) {
    $HTMLDIR="/www/doc/tmp/BLAST_PAN/";
    $HTML_LOC="http://fasta.bioch.virginia.edu/tmp/BLAST_PAN/";
    $datafile="COUNT";

# get a new directory number
    increment_num($res_num = get_number($HTMLDIR.$datafile)+3);

    $res_dir = $HTMLDIR.$res_num."/";
    mkdir($res_dir, 0777) || die $!;
    $res_html = $HTML_LOC.$res_num."/";
}
else {$res_dir = "./";}

$entrez_url="http://www.ncbi.nlm.nih.gov/htbin-post/Entrez/query?db=p&form=6&Dopt=f&html=no&uid=$gid";
$enb_url="http://www.ncbi.nlm.nih.gov/cgi-bin/Entrez/qserver.cgi?FORMAT=1&UID=";
$QUERY_LIMIT=60;

select(STDOUT), $| = 1;

print $q->header(),$q->start_html('blast_pan2 search'),
    $q->h3('blast_pan2 searching 1..2..3');

print "\n<pre>\n";

if ($opt_Debug) {
  print "Parameters are:\n";
    foreach $p_name ( $q->param() ) {
        $value = $q->param($p_name);
        $value =~ s/>/\&gt;/g;
        $value =~ s/</\&lt;/g;
        print "--> $p_name\t$value\n";
  }
}

$pgm = $q->param('PROGRAM');
$slib = $q->param('DATALIB');

if ($q->param('inputmode') eq 'input') {
    $query_list = $q->param('query_list');
}

if ($q->param('inputmode') eq 'upload') {
    $QFH = $q->upload('uploadfile');

    while (<$QFH>) {$query_list .= $_;}
    $query_list =~ s/\r/\n/g;
}

if ($q->param('inputmode') eq 'upload') {
    $QFH = $q->upload('uploadfile');

    while (<$QFH>) {$query_list .= $_;}
    $query_list =~ s/\r/\n/g;
    close $QFH;
}


# check query list for number of sequences ($QUERY_LIMIT max)
# break fasta db into individual sequences

@seq_lib = map { s/>$//s; s/^>?/>/s; $_; } split(/\n>/s, $query_list);

if (scalar @seq_lib > $QUERY_LIMIT )  {
    print 
	$q->h3("too many query sequences: ". &scalar(@seq_lib)." > $QUERY_LIMIT\n");
    print $q->end_html();
    exit();
}

$opt_sqid = $q->param('opt_sqid');
$opt_sqid &&= "--sqid ". $opt_sqid;

if ($q->param('opt_id_cut') eq 'checked') {
    $opt_id_cut = "--id_cut";
}
else {$opt_id_cut = ""; }


if ($q->param('CONFIRM') eq 'checked') {

    $conf_db = $res_dir . $q->param('uploaddata');
    unless (open (CFH,">$conf_db")) {
	print $q->h3("cannot open $conf_db");
	print $q->end_html();
	exit();
    }

    $QFH = $q->upload('uploaddata');
    if ( $QFH ) {
	while (<$QFH>) {
	    $_ =~ s/\r/\n/g;
	    print CFH $_;
	}
	close $QFH;
	close CFH;

	$conf_db = "--my_lib " . $conf_db;

#	print STDERR "using $conf_db\n";
    }
    else {
	print STDERR $q->param('uploaddata')," undefined\n";
	$conf_db = "";
    }
}
else {$conf_db = "";}

$bl_org = $q->param('GI_LIST');
$bl_org ||= "";
$bl_org &&= "--bl_org $bl_org";

$bl_org_text = $q->param('LIST_ORG');
$bl_org_text ||= "";
$bl_org_text &&= "--bl_org_text $bl_org_text";

$bp_args = "--slib $slib --pgm $pgm --h_dir $res_dir --r_dir $res_dir $bl_org $bl_org_text $conf_db $opt_sqid $opt_id_cut";

#print STDERR 
#"executing /home/wrp/fa_cvs/fp_00/blast_panl2.pl --slib $slib --pgm $pgm\n";
#print STDERR "$query_list\n";

open (BP_FH,"| /home/wrp/fa_cvs/fp_00/fast_panl3.pl $bp_args");
print BP_FH "$query_list\n";
close BP_FH;

print "</pre>\n";

print "<HR><P>\n";
print $q->h3("The pdf file is at <A HREF=\"${res_html}blast_pan.pdf\"> ${res_html}blast_pan.pdf</A>");
print "<P><HR>\n";

print $q->end_html();
print "\n";

exit();

sub get_number {
    my $file = shift;
    my $num;

    $LOCK_SH = 1;
    $LOCK_EX = 2;
    $LOCK_NB = 4;
    $LOCK_UN = 8;

    open(NUMBER,"<$HTMLDIR/$datafile")||die $!;
    flock(NUMBER,$LOCK_EX);
    $num = <NUMBER>;
    flock(NUMBER,$LOCK_UN);
    close(NUMBER);

    return $num;
}

sub increment_num {
    my $num = shift;
    open(NUMBER,">$HTMLDIR/$datafile")||die $!;
    flock(NUMBER,$LOCK_EX);
    print NUMBER "$num"; 
    flock(NUMBER,$LOCK_UN);
    close(NUMBER);
}
