#!/usr/bin/perl -w
#
#  orgininal file - do not use 
#
#   $Name: HEAD $ - $Id: fast_pan.pl,v 1.3 2000/10/15 17:47:12 wrp Exp $
#
$ver_str = "fast_pan.07.1 3-June-2000";
#
# fast_pan.pl
#
# copyright (1998) William R. Pearson and the U. of Virginia
#
# combine a set of tfastx3 results (against $DEF_LIB = 'c' library), plotting
# panels with bar heights indicating sequence E()-value, color showing
# the position of the overlap on the query sequence,
# and a horizontal line indicating percent identity. The percent identity
# (right y-axis) is scaled to highlight percent identities from 70 - 95%
#
# usage: fast_pan.pl [--options] name_file
#
# options:
#	--h_dir	set the directory for the html files (def = ".")
#	--r_ext	set the search_file.extension (def = "tx2")
#	--r_dir	set the search_file directory (def = ".")
#	--t_dir  directory for temporary files
#
#	--post	postscript results file ("fast_pan.ps")
#		(also used for pdf)
#       --ps_head header string for postscript output
#
#	--nosearch  do not do tfastxy3 search if results not found
#	--slib	library to search if results not found (def = "c" for est_mouse)
#	--mat	scoring matrix (BL50 by default, use MD20 for orthologs)
#	--gaps	string of gap penalties and options
#
#	--minh	min pairwise E() value (0.01)
#	--mint	min sum -log(E)  (4)
#	--col	color scheme 0,1,2   (2)
#	--idcut	do not show results if %id > sqid (false)
#	--sqid	show "*" if %id > sqid (0.95)
#
#	--relib	library to be re-searched from WWW link ($DEF_RELIB = "q")
#       --my_lib local library for re-search  (if not specified,
#	         use $ENV{"MY_LIBRARY}
#
#       --verbose  print additional progress information
#
# not options, but will change for different problems:
# program: tfastx3_t
#
# this program requires a working "sort" program that will sort on
# selectable primary and secondary keys
#

use Getopt::Long;

$ref_str = "Retief, J. D.,  Lynch, K. R., and Pearson, W. R., (1999) Genome Res. 9:373-382\n";

##################################################################
## -- edit this http link line:
##################################################################
$http_fasta_loc="<a HREF=\"http://fasta.bioch.virginia.edu/fasta/cgi/";

################################################################
# -- edit lines 
# -- $bin_dir	- specify the location of tfastx3, extractzp.pl
# -- $comp_pgm	- specify the name of the comparison program
# -- $comp_opts	- specify comparison program options
# -- $pdf_bin	- ps2pdf binary
#################################################################
$bin_dir="/seqprg/slib/bin/";

##################################################################
## -- edit line
## -- insert your name tfastx33 program
## -- specify "FASTLIBS" file location
## -- default library for search
##################################################################
$comp_pgm="tfastx33_t";
$comp_opts=" -N 10000 -H -w 80 -m 5 -E 0.1 -l /slib0/lib/fastlibs -Q ";
$pdf_bin = "/usr/local/bin/ps2pdf";
$DEF_LIB = "c";		# est_mouse on my system
$DEF_RELIB = "q";	# BLAST/swissprot on my system

################################################################
# start script
################################################################

$comp_bin=$bin_dir . $comp_pgm . $comp_opts;

$ln_10 = log(10);
$ll_scale = 50./$ln_10;

#foreach $n ( @ARGV ) {print $n;}
#print "\n\n";

if ($#ARGV < 0) { &print_use; die "\n quitting: input list not found\n\n";}

$opt_r_ext = $opt_r_dir = $opt_h_dir = $opt_post = $opt_t_dir =
    $opt_minh = $opt_mint = $opt_col = $opt_sqid = $opt_idcut = 
    $opt_relib = $opt_slib = $opt_mat = $opt_gaps = $opt_nosearch = 
    $opt_ps_head = "";

&GetOptions ("r_ext:s","r_dir:s","h_dir:s","t_dir:s","out:s","post:s","minh:f","mint:f","col:f","sqid:f","idcut","relib:s","nosearch","slib:s","mat:s","gaps:s","my_lib:s","ps_head:s","verbose");

###Set the default output file name
#  make certain it starts with "."
if (! $opt_r_ext) {$opt_r_ext = ".tx2";}
if ($opt_r_ext !~ /^\./) {$opt_r_ext = "." . $opt_r_ext;}

###Set the default output file name
if (! $opt_r_dir) {$opt_r_dir = "./";}
elsif ($opt_r_dir !~ /\/$/) { $opt_r_dir .= '/';}

###Set the default output file name
if (! $opt_h_dir) {$opt_h_dir = "./";}
elsif ($opt_h_dir !~ /\/$/) { $opt_h_dir .= '/';}
if ($opt_h_dir =~ /^~/) { $opt_h_dir =~ s/^~/$ENV{HOME}/;}

###Set the default output file name
if ( ! $opt_t_dir) {$opt_t_dir = "./";}
elsif ($opt_t_dir !~ /\/$/) { $opt_t_dir .= '/';}
if ($opt_t_dir =~ /^~/) { $opt_t_dir =~ s/^~/$ENV{HOME}/;}

###Set the default output file name for the postscript file
if (! $opt_post ) {$opt_post = "fast_pan.ps";}

###Set the default minimum hit E() value
if (! $opt_minh ) {$opt_minh = "0.01";}

###Set the default query hit E() value
if (! $opt_mint ) {$opt_mint = "4";}

###Set the default threshold for identity "*"
if (! $opt_sqid ) {$opt_sqid = "95.0";}
$sqi_cut = 100.0 - sqrt(100.0 - $opt_sqid)*10.0;

if (! $opt_idcut ) {$id_cut = 0;}
else {$id_cut = 1;}

### set library for re-search
if (! $opt_relib ) {$def_lib = $DEF_RELIB;}
else {$def_lib = $opt_relib;}

### set library for initial search
if (! $opt_slib ) {$slib = $DEF_LIB;}
else {$slib = $opt_slib;}

if (! $opt_my_lib ) {$my_lib = $ENV{"MY_LIBRARY"};}
else {$my_lib = $opt_my_lib;}

if ( $opt_mat ne "") {$opt_mat = "-s ".$opt_mat;}

###Set the color table
if (!$opt_col) {$opt_col = 2;}
if ($opt_col > 2) {$opt_col = 0;}

$l_color ="/l_color { [ 0.2  0.2  0.2  0.2  0.2  0.2  0.2  0.2  ] } def";

if ($opt_col == 0) {
    $red =   "/r_color { [ 0.2 0.8 0.2 0.8 0.2 0.8 0.2 0.8 ] } def";
    $green = "/g_color { [ 0.2 0.8 0.2 0.8 0.2 0.8 0.2 0.8 ] } def";
    $blue =  "/b_color { [ 0.2 0.8 0.2 0.8 0.2 0.8 0.2 0.8 ] } def";
} #close if ($opt_col == 0)

if ($opt_col == 1) {
    $red =  "/r_color { [ 0.0 0.0 0.2 0.0 0.5 0.50 1.00 1.0 ] } def";
    $green ="/g_color { [ 0.0 0.0 0.2 1.0 0.5 0.25 0.33 0.0 ] } def";
    $blue = "/b_color { [ 0.0 1.0 0.8 0.0 0.0 0.0  0.00 0.0 ] } def";
} #close if ($opt_col == 1)

if ($opt_col == 2) {
    $red = "/r_color { [ 0.0 0.0 0.0 0.0 1.0 1.0 1.0 0.5 ] } def";
    $green = "/g_color { [ 0.0 0.0 1.0 1.0 0.0 0.0 1.0 0.5 ] } def";
    $blue = "/b_color { [ 0.0 1.0 0.0 1.0 0.0 1.0 0.0 0.5 ] } def";
} #close if ($opt_col == 2)


#########################################################################
###Load all the query sequences in an array
#########################################################################
$n = 0;

print STDERR "\n**** $ver_str - starting ",`date`,"\n";
print STDERR "please cite:\n $ref_str\n\n";

while (<>) {

    chomp;

    $line = $_;

### check for spacer
    if (/^-/) {
	push(@search_seq,"-");
	push(@search_lab," ");
	push(@color,"0");
	$n++;
	next;
    }

### Remove paths

    ($search_seq,$color,$seq_cut)= ($line =~ /^(\S+)\s*(\d+)\s*(\d*\.*\d*)$/);
    $search_q = $search_seq;

    @tmp = split("/",$search_seq);
    $search_seq = $tmp[$#tmp];
    if ($search_seq eq "") {$search_seq = $search_q;}

#    print ">$search_q:$search_seq:$color:$seq_cut:\n";

    if ((length $search_seq) == 0) {next;}
    $search_seq =~ s/\.aa//;
    push (@search_seq,$search_seq);
    $q_lab = $search_seq;
    $q_lab =~ s/\..*$//;

    if ($seq_cut eq "") {$seq_cut = $opt_sqid;}
    if ($seq_cut <= 1.0) {$seq_cut *= 100.0;}
    push (@search_lab,$q_lab);
    push (@color,$color);
    $cut_arr{$search_seq} = $seq_cut;
    $col_arr{$search_seq} = $color;
    $srch_q_arr{$search_seq} = $search_q;
    $query_lab{$search_seq} = $q_lab;
    if ($opt_verbose) {
	print STDERR "|$search_seq|$color|$seq_cut|$search_q|$line|\n";
    }
    $n++;
} #close while (<>)

###Set the histogram scale

$xh = 9;
$xr = 10;
$r_margin=72;

if ($n < 40) {$xh = 10; $xr = 12;}
if ($n > 50) {$xh = 8; $xr = 9; $r_margin=48}

$q_total = $n;

#########################################################################
### Do the search if flag is set
#########################################################################

foreach $search_seq (@search_seq) {

    if ($search_seq eq "-"  ) { next;}

    $search_flag = 0;

#########################################################################
### Check to see if all the output files exist and set a flag
#########################################################################

    $search_res = $opt_r_dir . $search_seq . $opt_r_ext;
    if ($opt_verbose && $search_res ne "") {
	print STDERR "checking $search_res\n";
    }

    $f_name = $search_res;
    $fa_name = $opt_r_dir.$search_seq.".aa".$opt_r_ext;
    if (!$opt_nosearch && ! -f $f_name && ! -f $fa_name ) {
	if ($opt_verbose) {
	    print STDERR "I cannot find $search_res (or $fa_name) so I will do a search\n";
	}
	$search_flag = 1;

    } #close if (-r $search_seq$opt_r_ext)
    if (-f $fa_name) {rename($fa_name,$f_name);}

    if ($search_flag == 1) {
	$search_q = $srch_q_arr{$search_seq};

################################################################
## -- edit these lines for your program location
################################################################
	if ($opt_verbose) {
	    print STDERR "executing $comp_bin $opt_mat $opt_gaps $search_q $slib > $search_res\n";
	}
	system ("$comp_bin $opt_mat $opt_gaps $search_q $slib > $search_res");

    } #close if ($search_flag == 1)

} #close foreach $search_seq (@search_seq)

#########################################################################
### Read the accession numbers in the output files
#########################################################################

$fast_tmp0 = $opt_t_dir . "fast_tmp.tmp";
open (TEMP, ">$fast_tmp0") || die "cannot open $fast_tmp0";

$q_num = 0;

foreach $search_seq (@search_seq) {

    if ($search_seq eq "-") {next;}
    $f_name = $opt_r_dir.$search_seq.$opt_r_ext;
    if (open (SEARCH, "< $f_name")==0) {
	print STDERR "$f_name not found 1\n";
	next;
    }

    if ($opt_verbose) {print STDERR "Scanning $f_name\n";}

    $q_name = $search_seq;

    # get the query sequence length:

    while (<SEARCH>) {
	if (/$search_seq: *(\d+) aa\s*$/ || /$search_seq\.aa: *(\d+) aa\s*$/) {
	    $q_len = $1;
	    last;
	}
    }

    $l_pos = tell(SEARCH);
    while (<SEARCH>) {
	if (!/^>>/) { 
	    $l_pos = tell(SEARCH);
	    next;
	}  # skip down to alignments

	if (/^>>>/) { 
	    $l_pos = tell(SEARCH);
	    next;
	}
#
#	here we have the alignment description line, which may
#       have a genbank locus name (from a genbank flat file with GCG) 
#	or it may have a "gi|" number from est_human or est_mouse.
#
#	check for "gi|" number, otherwise get locus
#
#	print "$_";
	if (/^>>gi\|/) {($l_name,$line) = /^>>(gi\|\w+)\|(.+)/;}
#
# -- edit this line for unusual locus identifiers.
# this line is included for people who use ':' instead of '|' as their genbank
# description separator.  If the program does not produce any output, this 
# part of the code is likely to be the culprit.
#
	elsif (/^>>gb:/) {($l_name,$line) = /^>>(gb:\w+)\|(.+)/;}
	else {($l_name,$line) = /^>>(\w+)\s(.+)/;}

	($l_len) = /\( *(\d+) aa\)$/;

	$_ = <SEARCH>;

	if (/^rev-comp/) {$l_name = $l_name.'_r'; }

	($e_val) = /expect\(\) *(\d.*)$/;
# if "expect()" not found (fasta33t02), then look for "E()" 
	if ( !defined $e_val) {($e_val) = /E\(\): *(\d.*)$/;}

#	print STDERR "$e_val\n";

	if ($e_val > 0.0) {$le_val = -log($e_val)/$ln_10;}
	else {$le_val = 100.0;}
	if ($le_val > 100.0 ) {$le_val = 100.0;}
	$tot_e = $le_val;


#	print "$e_val\n";
	$_ = <SEARCH>; # Smith-Waterman
	$ovrlap = 0;

	($ident) = /;\s+(\d+.\d+)% identity/;
	($g_ident) = /identity \((\d+.\d+)% ungapped\)/;
	if (defined $g_ident) {$ident = $g_ident;}
#
#  get alignment coordinates
#
	($ovrlap,$start0,$stop0,$start1,$stop1) =
	    / in (\d+) aa overlap \((\d+)-(\d+):(\d+)-(\d+)\)$/;

	if (int($start1/1000) > 0) {$l_name .= "_".int($start1/1000)."K";}

#	print "++$l_name\t$l_len\t$tot_e\n";

	$l_tot_e{$l_name} += $tot_e;


#
#  reverse them if start > stop 
#
	if ($start1 > $stop1 ) {($start1,$stop1) = ($stop1,$start1);}

	$_ = <SEARCH>; # blank
	if (/^<A HREF/) {	# check for -m 6 file 
	    $_ = <SEARCH>;
	    if ($_ =~ /<p>/) {  $_ = <SEARCH>;}
	} 
	$_ = <SEARCH>; # >$hit ##- ##:

	if (/^>\S+\s+(\d+)- *(\d+):/) {
	    $startq = $1;
	    $stopq = $2;
	    if ($stopq > $q_len) {$stopq = $q_len;}
	}
	else { $startq = 0; $stopq = $q_len};

	if ($e_val < $opt_minh) {
	    printf TEMP "%s\t%d\t%s\t%d\t%.2f\t%d\t%d\t%.2f\t%d\t%d\t%d\t%d\t%.2f\t%s\n",
	    $q_name,$q_num,$l_name,$l_pos,$tot_e,$q_len,$l_len,$le_val,$start0,$stop0,$start1,$stop1,$ident,$line;
	}
	$l_pos = tell(SEARCH);
    } #  close while (<SEARCH>)

    $q_num++;

    close (SEARCH);
} #close foreach $search_seq (@search_seq)

print TEMP "ZZZZZZZZZZZZ\t99\tZZZZZZZZZZZZ\t-99\n";
close (TEMP);

# Now re-read the fast_tmp.tmp file, but write out $tot_e with the
# correct values

open(TEMP, $fast_tmp0) || die "Cannot open $fast_tmp0\n";

$fast_tmp2 = $opt_t_dir . "fast_tmp.tmp2";
$fast_tmps = $opt_t_dir . "fast_tmp.srt";

open(TEMP2,"> $fast_tmp2") || die "Cannot open $fast_tmp2\n";

while (<TEMP>) {
    if (/^ZZZZZZZZZZ/) { last;}
    chop();
    ($q_name,$q_num,$l_name,$l_pos,$tot_e,$q_len,$l_len,$le_val,$startq,$stopq,$start1,$stop1,$ident,$line) =
	split('\t');

    $tot_e = $l_tot_e{$l_name};
#
# only print it out if tot_e > mint
#
    if ($tot_e >= $opt_mint) {
	printf TEMP2 "%s\t%d\t%s\t%d\t%.2f\t%d\t%d\t%.2f\t%d\t%d\t%d\t%d\t%.2f\t%s\n",
	$q_name,$q_num,$l_name,$l_pos,$tot_e,$q_len,$l_len,$le_val,$startq,$stopq,$start1,$stop1,$ident,$line;
    }
}

close TEMP;
print TEMP2 "ZZZZZZZZZZZZ\t99\tZZZZZZZZZZZZ\t-99\n";
close TEMP2;

# fields are:
#   1       2      3       4      5      6      7      8       9       10     11      12     13     14
#  $q_name $q_num $l_name $l_pos $tot_e $q_len $l_len $le_val $startq $stopq $start1 $stop1 $ident $line;
#
# Now sort based on l_name, l_pos, q_num (numeric).  On Digital Unix, the
# command is:  sort -b -k 3,3 -k 11,11n -k2,2n < fast_tmp.tmp > fast_tmp.srt
#

system("sort -b -t '\t' -k5,5nr -k3,3 -k 11,11n $fast_tmp2 > $fast_tmps");

#
# In this test version, we will simply output the query sequence if it
# meets the E()_total criterion as it is encountered - thus in alphabetical
# order.  In the next version, we may scan twice - once to get e()_total
# and a second time to actually output the values
# 

open (TEMP,"$fast_tmps") || die "cannot open $fast_tmps\n";

open (POST, ">$opt_post") || print STDERR "Cannot open $opt_post\n";

$l_count = 0; #counter for hits
$page = 1;

&header;
$ps_dcnt = 0;

$l_old = $line_old = "";
$tot_e = 0;

foreach $s_name ( @search_seq ) {
    $val_arr{$s_name}[0] = -1;
    $val_arr{$s_name}[5] = 0.0;
}

#
#  This section reads in the final sorted list of results, of the form:
#  q_name   color  l_name   l_pos tot_e  q_len l_len le_val startq stopq start1 stop1 %id   line:
#  gta1_human  0  gi|972123 50367 253.27  222   375   3.05    7    117   13     363   32.77 
#  gta2_human  1  gi|972123 45471 253.27  222   375   3.17    7    117   13     363   33.61
#
#  The lines are read, waiting for a change in the l_name field which indicates
#  a new library sequence.  Until then, the results are stored in @val_arr{},
#  hashed by $q_name.  This allows the results to be read out in the correct
#  order when it comes time to write the postscript panel
#
#  To display the html file in E()-value order, rather than postscript display
#  order, we need to sort the results according to e-value ($val_arr{}[8]).
#  and then write out the html file highest e-value first.
#

L1:
while (<TEMP>) {
    chop;
    ($q_name,$q_num,$l_name,$l_pos,$tot_e,$q_len,$l_len,$le_val,$startq,$stopq,$start1,$stop1,$ident,$line) =
	split('\t');
#    print ">> $q_name: $le_val, $startq, $stopq, $q_len, $ident\n";
    if ($l_name eq $l_old ) {	# same name as before
	$val_arr{$q_name}[0] = $le_val;
	$val_arr{$q_name}[1] = $startq;
	$val_arr{$q_name}[2] = $stopq;
	$val_arr{$q_name}[3] = $q_len;
	$val_arr{$q_name}[4] = $l_pos;
	$val_arr{$q_name}[5] = $ident;
	$val_arr{$q_name}[6] = $start1;
	$val_arr{$q_name}[7] = $stop1;
	$val_arr{$q_name}[8] = $le_val;
    }
    else {
#
#  We have a new library sequence name, so we must write everything out for
#  the old library sequence
#
	if ($l_old ne "") {
	    $skip_flag = 0;
#  if I have an %id cutoff, check whether any query meets it
	    if ($id_cut) {
		foreach $s_name ( @search_seq ) {
		    if ($s_name eq "-" || $val_arr{$s_name}[0] < 0) {next;}
		    if ($val_arr{$s_name}[5] > $cut_arr{$s_name}) {
			$skip_flag = 1;
			last;
		    }
		}
	    }

#  if we aren't skipping because of %id, check formatting
	    if ($skip_flag != 1) {
		if ( ($l_count % 5) == 0) {	# print labels
		    &page;
		}
		&panel;
		$ps_dcnt++;
#		print ">-lname: $l_old: $line_old\n";
		print POST "%% lname: $l_old: $line_old\n";
#
#  $lt_name is used both for html and postscript, so set it here
#

		$lt_name = $l_old.".html";
		($lt_acc) = ($line_old =~ /^\s*([\w\.\|]+)\s+/); 
		if (!defined($lt_acc)) {$lt_acc = "";}

#
#  now start writing the postscript panel
#
#  needed to set sequence limits for re-search
#
		$min_start1 = 1000000000; $max_stop1 = -1;

		foreach $s_name ( @search_seq ) {
		    if ($s_name eq "-") {
			print POST "( ) 0 0 0 1 0 hbox\n";
			next;
		    }
#
#  these variables are all $p_... 
#  so that the current values are not overwritten.
#
		    $p_le_val = $val_arr{$s_name}[0];
		    $p_startq = $val_arr{$s_name}[1];
		    $p_stopq = $val_arr{$s_name}[2];
		    $p_q_len= $val_arr{$s_name}[3];
		    $p_l_pos= $val_arr{$s_name}[4];
		    $p_ident = $val_arr{$s_name}[5];

		    if ($p_le_val >= 1) {
#
#  min_start1, max_stop1 is needed later for .html file
#  find it during this scan
#
			if ($val_arr{$s_name}[6] < $min_start1) {
			    $min_start1 = $val_arr{$s_name}[6];}
			if ($val_arr{$s_name}[7] > $max_stop1) {
			    $max_stop1 = $val_arr{$s_name}[7];}
#
#  set up postscript information
#
			$q_lab = $query_lab{$s_name};
			$q_file = $opt_r_dir.$s_name.$opt_r_ext;

			$lle_val = log($p_le_val)*$ll_scale;

			$ll_id = 100.0 - sqrt(100.0 - $p_ident)*10.0;
#
#	printf "%s %s = %.2f (%.4f) [%.3f] %3d %3d %3d %.2f %d\n",
#		$l_old,$s_name,$p_le_val,$lle_val,$ll_id,
#		$p_startq, $p_stopq, $p_q_len,$p_ident,
#		$col_arr{$s_name};
#
			if ($p_stopq > $p_q_len) { $p_stopq = $p_q_len; }
#
#  print the hbox command
#
			print POST "($lt_name#$q_lab) ";

			printf POST "%.4f %.2f %.2f %.2f %2d hbox\n",
			$lle_val, $ll_id, $p_startq/$p_q_len, $p_stopq/$p_q_len,
			$col_arr{$s_name};
#
#  at this point, we should be done with the postscript for one bar
#
		    }
		    else {
			print POST "( ) 0 0 0 1 0 hbox\n";
		    }
		}
#
#  start writing out the html file
#
		open(LTEMP,"> $opt_h_dir$lt_name") || die "cannot open $lt_name\n";
# put out html headers 
		print LTEMP "<html>\n<head>\n<title> $l_old matches </title>\n";
		print LTEMP "</head>\n<body>\n<h2>Query sequences matching:</h2>\n";
		print LTEMP "<b>$l_old $line_old</b>\n<hr>\n";

		if (! $p_l_len) {$p_l_len="";}
		if ($min_start1 < 0) {$min_start1 = 1;}
#
#  this code does not work if -N 5000 is used
#		if ($p_l_len ne "" && $max_stop1 > $p_l_len) {
#		    warn "max_stop1 ($max_stop1) out of range :$p_l_len:\n";
#		    $max_stop1 = $p_l_len;
#		}
#
		($lq_old) = ($l_old =~ /([A-Za-z\d]+)_*[rK\d]*/);
		print LTEMP "<p><h3>FASTX search with ".$http_fasta_loc."searchfa.cgi?query=$lq_old&db=n&pgm=fx&acc=$lt_acc&lib=$def_lib&start=$min_start1&stop=$max_stop1&n1=$p_l_len\">$l_old</a>&nbsp;&nbsp;".$http_fasta_loc."searchxf.cgi?query=$lq_old&acc=$lt_acc&db=n&lib=$def_lib&pgm=fx&start=$min_start1&stop=$max_stop1&n1=$p_l_len\">General&nbsp;research</a>  ";
		if ($my_lib) {
		    print LTEMP "Search:&nbsp;".$http_fasta_loc."searchfa.cgi?query=$lq_old&db=n&pgm=fx&acc=$lt_acc&opt_str=-z+11&lib=$my_lib&start=$min_start1&stop=$max_stop1&n1=$p_l_len\">$my_lib</a>";
		}
		print LTEMP "</h3>\n<hr>\n";
#
#  now jump into the data file and write out the alignments
#
#  sort sequence names by E()-value
#
		@evs_search_seq = 
		    sort {$val_arr{$b}[0] <=> $val_arr{$a}[0]} @search_seq;
#
#  display alignments from most to least significant
#
		foreach $s_name ( @evs_search_seq ) {

		    $p_le_val = $val_arr{$s_name}[0];
		    $p_startq = $val_arr{$s_name}[1];
		    $p_stopq = $val_arr{$s_name}[2];
		    $p_q_len= $val_arr{$s_name}[3];
		    $p_l_pos= $val_arr{$s_name}[4];
		    $p_ident = $val_arr{$s_name}[5];

		    if ($p_le_val >= 1) {
			$q_lab = $query_lab{$s_name};
			$q_file = $opt_r_dir.$s_name.$opt_r_ext;

			open(QTEMP,"$q_file") ||
			    print STDERR "cannot open q_file: $q_file\n";
#			print "seeking to $p_l_pos in $q_file\n";
			seek(QTEMP,$p_l_pos,0);
			print LTEMP "<A name=$q_lab>\n";
			print LTEMP
			    "<b>Match to $q_lab ($p_q_len aa)</b><br>\n<tt><pre>\n";
			$_ = <QTEMP>; 
#			print $_;

			print LTEMP $_;
			while (<QTEMP>) {
			    if (/^<\/pre>/i || /^>>/ || /residues in/) {
				print LTEMP "</pre></tt>\n";
				print LTEMP "<hr>\n";
				last;
			    }
			    print LTEMP $_;
			}
			close(QTEMP);
#
#  reset $le_val to make certain it is not used by the next
#  panel unless it is really specified
#
			$val_arr{$s_name}[0] = -1;
		    }
		} # $s_name
#
#  here we are done with all of the query sequences
#
#
#  finish the HTML and close it
#
		print LTEMP "<p>\n</body>\n</html>\n";
		close(LTEMP);
#
#  finish the postscript panel
#
		print POST "y2axis\n";
		print POST "% label the panel with EST name\n";
		print POST "18 -12 moveto\n";
		print POST "gsave\n";
		print POST "/Helvetica findfont sl scalefont setfont\n";
		$line_old =~ s/\(//g;
		$line_old =~ s/\)//g;
		$line_old = substr($line_old,0,60);
		print POST "($l_old $line_old) hprint\n";
		print POST "grestore\n\n";
#
# now point to the results page
#
		$urx_pos = $q_total * $xr;
		print POST "[ /Rect [ 0 0 $urx_pos 100 ]\n";
		print POST "/Action << /Subtype /URI /URI ($lt_name) >>\n";
		print POST "/Subtype /Link\n/ANN pdfmark\n";

		if (($l_count % 5) == 4) {
		    print POST "showpage\n";
		    $page++;
		}
		else {
		    print POST "0 -135 translate\n";
		}
		$l_count++;
	    } # skip flag
#
#  all done with this library sequence
#
	    else {
		print STDERR "skipping $l_old:\t";
		foreach $s_name ( @search_seq ) {
		    if ($s_name eq "-" || $val_arr{$s_name}[0]<0) {next;}
		    $p_ident=$val_arr{$s_name}[5];
		    if ($p_ident > $cut_arr{$s_name}) {
			print STDERR "$s_name: $p_ident\t";
		    }
#
#  even if we skip, we must reset some variables
#
		    $val_arr{$s_name}[0] = -1;
		    $val_arr{$s_name}[5] = 0.0;
		}
		print STDERR "\n";
	    } # ! else skip_flag
	} # l_old eq ""

	if (/^ZZZZZZZZZZZZ/) { last; }
#
#  we have now displayed the results for one library sequence
#  load the values for the next new library sequence
#
	$tot_e = $le_val;
	$val_arr{$q_name}[0] = $le_val;
	$val_arr{$q_name}[1] = $startq;
	$val_arr{$q_name}[2] = $stopq;
	$val_arr{$q_name}[3] = $q_len;
	$val_arr{$q_name}[4] = $l_pos;
	$val_arr{$q_name}[5] = $ident;
	$val_arr{$q_name}[6] = $start1;
	$val_arr{$q_name}[7] = $stop1;
	$val_arr{$q_name}[8] = $le_val;
	$p_l_len = $l_len;
	
	$l_old = $l_name;	# update the name
	$line_old = $line;
    } # end else ($l_old eq $l_name) 
}
if (($l_count - 1)%5 != 4) {print POST "showpage\n";}

if ($ps_dcnt == 0) {&page; &nh_panel;}

&footer;

close(POST);

$opt_pdf = $opt_post;
$opt_pdf =~ s/.ps$/.pdf/;
print STDERR "writing $opt_pdf\n";
system("$pdf_bin $opt_post $opt_pdf");
if ($opt_h_dir ne "./") {
    print STDERR " and moving $opt_pdf to $opt_h_dir$opt_pdf\n";
    system("mv $opt_pdf $opt_h_dir$opt_pdf");
}
#################################################################
# clean up after ourselves
#################################################################
#unlink ($opt_post,"$fast_tmp0","$fast_tmp2","$fast_tmps");

print STDERR "\n**** $ps_dcnt hits found\n";
print STDERR "**** $ver_str - finished ",`date`,"\n";

################################################################
################################################################

sub header {

    $date = `date`;

    print POST <<"POST_EOF";
%!PS-Adobe-2.0
%%Creator: fast_pan.pl
%%Title: $opt_post
%%CreationDate: $date
%%PageOrder: Ascend
%%EndComments
%
% disable PDFmark if it is not present
%
/pdfmark where
{pop} {userdict /pdfmark /cleartomark load put} ifelse

% - the general link method for pdfmark
%  [ /Rect [ llx lly urx ury ]
%    /Action << /Subtype /URI /URI (http://www.adobe.com) >>
%    /Subtype /Link
%    /ANN
%   pdfmark
%
%
% postscript to display histogram profiles of EST's
%
% scale/size variables
%
/ys 1 def       % y-scaling \(1 pt = 1 y-unit\)
/xh $xh def       % histogram bar width
/xr $xr def       % spacing between labels, histogram bars
/xo 0 def % xstart position (0,?) for plot, must be reset to 0 each time
/yo 0 def % ystart position (?,0) for plot, must be reset to 0 each time
/sz $xh def       % size of diagonal label text
/szd 3  def     % diagonal offset
/sl 10 def      % size of EST plot label
$red
$green
$blue
$l_color

/yaxis {
        0 0 moveto
        0 100 lineto
        stroke
	-30 25 moveto
	(Log(Expect)) vprint
        newpath
        0 15.05 moveto
        -3 0 rlineto gsave stroke grestore
        (-2) dup stringwidth pop neg -3 add -5 rmoveto show
        0 34.95 moveto
        -3 0 rlineto gsave stroke grestore
        (-5) dup stringwidth pop neg -3 add -5 rmoveto show
        0 50 moveto
        -3 0 rlineto gsave stroke grestore
        (-10) dup stringwidth pop neg -3 add -5 rmoveto show
        0 65.05 moveto
        -3 0 rlineto gsave stroke grestore
        (-20) dup stringwidth pop neg -3 add -5 rmoveto show
        0 84.9 moveto
        -3 0 rlineto gsave stroke grestore
        (-50) dup stringwidth pop neg -3 add -5 rmoveto show
        0 100 moveto
        -3 0 rlineto gsave stroke grestore
        (-100) dup stringwidth pop neg -3 add -5 rmoveto show
        } def
%
%
/y2axis {
        xo 0 moveto
        0 100 rlineto
        stroke
	xo 30 add 35 moveto
	(% ident) vprint
        newpath
        xo 0 moveto
        3 0 rlineto gsave stroke grestore
        (0) 1 -5 rmoveto show
        xo 10.56 moveto
        3 0 rlineto gsave stroke grestore
        (20) 1 -5 rmoveto show
        xo 22.54 moveto
        3 0 rlineto gsave stroke grestore
        (40) 1 -5 rmoveto show
        xo 36.75 moveto
        3 0 rlineto gsave stroke grestore
        (60) 1 -5 rmoveto show
        xo 55.28 moveto
        3 0 rlineto gsave stroke grestore
        (80) 1 -5 rmoveto show
        xo 68.38 moveto
        3 0 rlineto gsave stroke grestore
        (90) 1 -5 rmoveto show
        xo 77.64 moveto
        3 0 rlineto gsave stroke grestore
        (95) 1 -5 rmoveto show
        xo 90.00 moveto
        3 0 rlineto gsave stroke grestore
        (99) 1 -5 rmoveto show
        xo 100 moveto
        3 0 rlineto gsave stroke grestore
        (100) 1 -5 rmoveto show
        } def
%

%
% (label) -LogE f_start f_end color -Log(id) hbox
%
/hbox {
        /c exch def
        /sp exch def
        /st exch def
        sp 1.0 ge { /sp 1.0 def } if
        st 0.0 le { /st 0.0 def } if
	/sqi exch def
        /as exch def
	/urlstr exch def
        /sd sp st sub as mul ys mul def
        sd 2 lt { /sd 1 def } if

        as 0 gt {
         gsave
         c                       % [c]
         dup r_color             % [c][c] [r_color]
         exch                    % [c][r_color][c]
         get                     % [c][r]
         exch                    % [r][c]
         dup b_color exch get exch  % get g_color
         g_color exch get exch  % get b_color
         setrgbcolor
         newpath
         xo st as mul ys mul  moveto
         0 sd rlineto
         xh 0 rlineto
         0 sd neg rlineto
         closepath
         fill
         stroke
         newpath		% write horizontal line for % ident
         l_color c get setgray	% get line color (l_color)
         xo 1.5 add sqi ys mul yo add moveto
         xh 3 sub 0 rlineto
	 1.5 setlinewidth
         stroke

	 sqi $sqi_cut gt { xo xh 4 div add 96 moveto 
		/Helvetica findfont 18 scalefont setfont (*) hprint } if

         grestore
        } if
        gsave
        1.25 setlinewidth
        newpath
         xo yo moveto
         0 as ys mul rlineto
         xh 0 rlineto
         0 as neg ys mul rlineto
         closepath

%	 pathbbox 
%	 /rury exch def
%	 /rurx exch def
%	 /rlly exch def
%	 /rllx exch def

         stroke
         grestore

%	[ /Rect [ rllx rlly rurx rury ]
%	/Action << /Subtype /URI /URI urlstr >>
%	/Subtype /Link
%	/ANN
%	pdfmark

        /xo xo xr add def
} def

% header for all data
%
/right { 0 rmoveto } def
/down { neg 0 exch rmoveto } def
/cm { 2.54 div 72 mul } def
/vprint { gsave 90 rotate show newpath stroke grestore } def
/dprint { gsave 45 rotate show newpath stroke grestore } def
/hrprint {
	gsave
	dup stringwidth pop neg 0 rmoveto
	show 
	grestore
	} def

/dprintr {
        gsave
        dup r_color exch get exch  % get g_color
        dup b_color exch get exch  % get g_color
        g_color exch get exch  % get g_color
        setrgbcolor
        45 rotate show
        newpath
        stroke grestore xr 0 rmoveto } def
/hprint { gsave show newpath stroke grestore } def
/nstr 7 string def
%%EndProlog

POST_EOF


} #close sub header

################################################################

sub page {

    print POST <<"POST_EOF";
%%Page: $page $page
/xo 0 def % xstart position (0,?) for plot, must be reset to 0 each time
/yo 0 def % ystart position (?,0) for plot, must be reset to 0 each time
/Helvetica findfont sz scalefont setfont
1 setlinewidth
% this section moves to the top of a page
% then moves to the top of a panel and labels the columns
$r_margin 600 translate
xh 2 div 120 moveto    % 0 120 move to top of panel

POST_EOF
    
    if ($opt_ps_head) {
	$l_margin = 612-2*$r_margin; # doubled because of translate above
	print POST "$l_margin 160 moveto\n";
	print POST "($opt_ps_head) hrprint\n";
	print POST "xh 2 div 110 moveto\n";
    }

    for ($i=0; $i <= $#search_seq; $i++ ) {
	if ($search_seq[$i]) {
	    print POST "($search_lab[$i]) $color[$i] dprintr\n";
	} 
    } 
    print POST "% labels are finished (use above section for each page)\n";

} #close sub page


################################################################

sub footer {

print POST <<"POST_EOF"
%%Trailer

%%EOF

POST_EOF

} #close sub footer

sub panel {

print POST <<"POST_EOF"
% put on the y-axis
/xo 0 def
/yo 0 def
0 0 moveto
% 0 100 lineto (done by yaxis)
yaxis
POST_EOF

}

sub nh_panel {

print POST <<"POST_EOF"
% put on the y-axis
/xo 0 def
/yo 0 def
120 40 moveto
/Helvetica findfont 24 scalefont setfont
(No matches found) show
showpage

POST_EOF
}

sub print_use {

print STDERR <<USE_EOF

 usage: fast_pan.pl [--options] name_file

 options (default):
	--h_dir	set the directory for the html files (".")
	--r_ext	set the search_file.extension ("tx2")
	--r_dir	set the search_file directory (".")
	--t_dir  directory for temporary files (".")

	--post	postscript results file ("fast_pan.ps")
		(also used for pdf)
        --ps_head header string for postscript output

	--nosearch  do not do tfastxy3 search if results not found
	--slib	library to search if results not found
	--mat	scoring matrix (BL50 by default, use MD20 for orthologs)
	--gaps	string of gap penalties and options

	--minh	min pairwise E() value (0.01)
	--mint	min sum -log(E)  (4)
	--col	color scheme 0,1,2   (2)
	--idcut	do not show results if %id > sqid (false)
	--sqid	show "*" if %id > sqid (0.95)

	--relib	library to be re-searched from WWW link
	  (\$ENV{\"DEF_RELIB\"} = "q")
        --my_lib local library for re-search  (if not specified, use
	  \$ENV{\"MY_LIBRARY\"})

        --verbose additional progress information
USE_EOF

}
